/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.engine.AbstractBentenGlossaryEnginePlugin;
import benten.ui.AbstractBentenUiPlugin;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * CAT 用語集 UI プラグインです。
 */
public class CatGlossaryUiPlugin extends AbstractBentenUiPlugin {

	/** 共用インスタンス */
	private static CatGlossaryUiPlugin plugin;

	final Map<String, AbstractBentenGlossaryEnginePlugin> fGlossaryMap = new HashMap<String, AbstractBentenGlossaryEnginePlugin>();

	private AbstractBentenGlossaryEnginePlugin fGlossaryEnginePlugin = null;

	/**
	 * コンストラクター。
	 */
	public CatGlossaryUiPlugin() {
	}

	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		loadExtensions();
	}

	@Override
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	/**
	 * このプラグインの共用インスタンスを取得。
	 * @return このプラグインの共用インスタンス
	 */
	public static CatGlossaryUiPlugin getDefault() {
		return plugin;
	}

	public void loadExtensions() {
		final AbstractBentenGlossaryEnginePlugin glossaryEngine = getGlossaryEnginePlugin();
		if (glossaryEngine != null) {
			fGlossaryEnginePlugin = glossaryEngine;
			glossaryEngine.load();
		}
	}

	/**
	 * このプラグインが保持している グロッサリー・エンジンを取得します。
	 *
	 * @return グロッサリー・エンジンのインスタンス。
	 */
	public BentenGlossaryEngine getGlossaryEngine() {
		return fGlossaryEnginePlugin.getEngine();
	}

	/**
	 * グロッサリー・エンジンのインスタンスを新規に作成します。
	 *
	 * 所定のルールに従って拡張ポイントを検索して グロッサリー・エンジンのインスタンスを作成します。
	 *
	 * @return グロッサリー・エンジンのインスタンス。
	 */
	public AbstractBentenGlossaryEnginePlugin getGlossaryEnginePlugin() {

		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.engine"); //$NON-NLS-1$
		log(point);
		if (point == null) {
			return null;
		}

		final IExtension[] extensions = point.getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			final IConfigurationElement[] cfgElems = extensions[i].getConfigurationElements();
			for (int j = 0; j < cfgElems.length; j++) {
				final IConfigurationElement cfgElem = cfgElems[j];
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					final Object obj = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (obj instanceof AbstractBentenGlossaryEnginePlugin) {
						fGlossaryMap.put(name, (AbstractBentenGlossaryEnginePlugin) obj);
					}
				} catch (final CoreException e) {
					log(e);
				}
			}
		}

		final IPreferenceStore store = BentenPreference.getStore();
		final String tmName = store.getString(Preference.GLOSSARY_NAME.name());
		return fGlossaryMap.get(tmName);
	}

	/**
	 * 用語集エンジン名のセットを取得します。
	 * @return 用語集エンジン名のセット
	 */
	public Set<String> getTmNameSet() {
		return fGlossaryMap.keySet();
	}
}
